SCHEMA_HELP = (
    "Structured data defines brand, contact, and storefront information to search "
    "engines. This information should be provided on the site’s root page (home page). "
    "If your organization has multiple locations, also provide this info on each "
    "location page using that particular location’s info."
)

# fmt: off
SCHEMA_ORG_CHOICES = (
    ("Organization", "Organization"),
    ("Airline", "Organization > Airline"),
    ("Corporation", "Organization > Corporation"),
    ("EducationalOrganization", "Organization > EducationalOrganization"),
    ("CollegeOrUniversity", "Organization > EducationalOrganization > CollegeOrUniversity"),
    ("ElementarySchool", "Organization > EducationalOrganization > ElementarySchool"),
    ("HighSchool", "Organization > EducationalOrganization > HighSchool"),
    ("MiddleSchool", "Organization > EducationalOrganization > MiddleSchool"),
    ("Preschool", "Organization > EducationalOrganization > Preschool"),
    ("School", "Organization > EducationalOrganization > School"),
    ("GovernmentOrganization", "Organization > GovernmentOrganization"),
    ("LocalBusiness", "Organization > LocalBusiness"),
    ("AnimalShelter", "Organization > LocalBusiness > AnimalShelter"),
    ("AutomotiveBusiness", "Organization > LocalBusiness > AutomotiveBusiness"),
    ("AutoBodyShop", "Organization > LocalBusiness > AutomotiveBusiness > AutoBodyShop"),
    ("AutoDealer", "Organization > LocalBusiness > AutomotiveBusiness > AutoDealer"),
    ("AutoPartsStore", "Organization > LocalBusiness > AutomotiveBusiness > AutoPartsStore"),
    ("AutoRental", "Organization > LocalBusiness > AutomotiveBusiness > AutoRental"),
    ("AutoRepair", "Organization > LocalBusiness > AutomotiveBusiness > AutoRepair"),
    ("AutoWash", "Organization > LocalBusiness > AutomotiveBusiness > AutoWash"),
    ("GasStation", "Organization > LocalBusiness > AutomotiveBusiness > GasStation"),
    ("MotorcycleDealer", "Organization > LocalBusiness > AutomotiveBusiness > MotorcycleDealer"),
    ("MotorcycleRepair", "Organization > LocalBusiness > AutomotiveBusiness > MotorcycleRepair"),
    ("ChildCare", "Organization > LocalBusiness > ChildCare"),
    ("Dentist", "Organization > LocalBusiness > Dentist"),
    ("DryCleaningOrLaundry", "Organization > LocalBusiness > DryCleaningOrLaundry"),
    ("EmergencyService", "Organization > LocalBusiness > EmergencyService"),
    ("FireStation", "Organization > LocalBusiness > EmergencyService > FireStation"),
    ("Hospital", "Organization > LocalBusiness > EmergencyService > Hospital"),
    ("PoliceStation", "Organization > LocalBusiness > EmergencyService > PoliceStation"),
    ("EmploymentAgency", "Organization > LocalBusiness > EmploymentAgency"),
    ("EntertainmentBusiness", "Organization > LocalBusiness > EntertainmentBusiness"),
    ("AdultEntertainment", "Organization > LocalBusiness > EntertainmentBusiness > AdultEntertainment"),
    ("AmusementPark", "Organization > LocalBusiness > EntertainmentBusiness > AmusementPark"),
    ("ArtGallery", "Organization > LocalBusiness > EntertainmentBusiness > ArtGallery"),
    ("Casino", "Organization > LocalBusiness > EntertainmentBusiness > Casino"),
    ("ComedyClub", "Organization > LocalBusiness > EntertainmentBusiness > ComedyClub"),
    ("MovieTheater", "Organization > LocalBusiness > EntertainmentBusiness > MovieTheater"),
    ("NightClub", "Organization > LocalBusiness > EntertainmentBusiness > NightClub"),
    ("FinancialService", "Organization > LocalBusiness > FinancialService"),
    ("AccountingService", "Organization > LocalBusiness > FinancialService > AccountingService"),
    ("AutomatedTeller", "Organization > LocalBusiness > FinancialService > AutomatedTeller"),
    ("BankOrCreditUnion", "Organization > LocalBusiness > FinancialService > BankOrCreditUnion"),
    ("InsuranceAgency", "Organization > LocalBusiness > FinancialService > InsuranceAgency"),
    ("FoodEstablishment", "Organization > LocalBusiness > FoodEstablishment"),
    ("Bakery", "Organization > LocalBusiness > FoodEstablishment > Bakery"),
    ("BarOrPub", "Organization > LocalBusiness > FoodEstablishment > BarOrPub"),
    ("Brewery", "Organization > LocalBusiness > FoodEstablishment > Brewery"),
    ("CafeOrCoffeeShop", "Organization > LocalBusiness > FoodEstablishment > CafeOrCoffeeShop"),
    ("FastFoodRestaurant", "Organization > LocalBusiness > FoodEstablishment > FastFoodRestaurant"),
    ("IceCreamShop", "Organization > LocalBusiness > FoodEstablishment > IceCreamShop"),
    ("Restaurant", "Organization > LocalBusiness > FoodEstablishment > Restaurant"),
    ("Winery", "Organization > LocalBusiness > FoodEstablishment > Winery"),
    ("GovernmentOffice", "Organization > LocalBusiness > GovernmentOffice"),
    ("PostOffice", "Organization > LocalBusiness > GovernmentOffice > PostOffice"),
    ("HealthAndBeautyBusiness", "Organization > LocalBusiness > HealthAndBeautyBusiness"),
    ("BeautySalon", "Organization > LocalBusiness > HealthAndBeautyBusiness > BeautySalon"),
    ("DaySpa", "Organization > LocalBusiness > HealthAndBeautyBusiness > DaySpa"),
    ("HairSalon", "Organization > LocalBusiness > HealthAndBeautyBusiness > HairSalon"),
    ("HealthClub", "Organization > LocalBusiness > HealthAndBeautyBusiness > HealthClub"),
    ("NailSalon", "Organization > LocalBusiness > HealthAndBeautyBusiness > NailSalon"),
    ("TattooParlor", "Organization > LocalBusiness > HealthAndBeautyBusiness > TattooParlor"),
    ("HomeAndConstructionBusiness", "Organization > LocalBusiness > HomeAndConstructionBusiness"),
    ("Electrician", "Organization > LocalBusiness > HomeAndConstructionBusiness > Electrician"),
    ("GeneralContractor", "Organization > LocalBusiness > HomeAndConstructionBusiness > GeneralContractor"),
    ("HVACBusiness", "Organization > LocalBusiness > HomeAndConstructionBusiness > HVACBusiness"),
    ("HousePainter", "Organization > LocalBusiness > HomeAndConstructionBusiness > HousePainter"),
    ("Locksmith", "Organization > LocalBusiness > HomeAndConstructionBusiness > Locksmith"),
    ("MovingCompany", "Organization > LocalBusiness > HomeAndConstructionBusiness > MovingCompany"),
    ("Plumber", "Organization > LocalBusiness > HomeAndConstructionBusiness > Plumber"),
    ("RoofingContractor", "Organization > LocalBusiness > HomeAndConstructionBusiness > RoofingContractor"),
    ("InternetCafe", "Organization > LocalBusiness > InternetCafe"),
    ("LegalService", "Organization > LocalBusiness > LegalService"),
    ("Attorney", "Organization > LocalBusiness > LegalService > Attorney"),
    ("Notary", "Organization > LocalBusiness > LegalService > Notary"),
    ("Library", "Organization > LocalBusiness > Library"),
    ("LodgingBusiness", "Organization > LocalBusiness > LodgingBusiness"),
    ("BedAndBreakfast", "Organization > LocalBusiness > LodgingBusiness > BedAndBreakfast"),
    ("Campground", "Organization > LocalBusiness > LodgingBusiness > Campground"),
    ("Hostel", "Organization > LocalBusiness > LodgingBusiness > Hostel"),
    ("Hotel", "Organization > LocalBusiness > LodgingBusiness > Hotel"),
    ("Motel", "Organization > LocalBusiness > LodgingBusiness > Motel"),
    ("Resort", "Organization > LocalBusiness > LodgingBusiness > Resort"),
    ("ProfessionalService", "Organization > LocalBusiness > ProfessionalService"),
    ("RadioStation", "Organization > LocalBusiness > RadioStation"),
    ("RealEstateAgent", "Organization > LocalBusiness > RealEstateAgent"),
    ("RecyclingCenter", "Organization > LocalBusiness > RecyclingCenter"),
    ("SelfStorage", "Organization > LocalBusiness > SelfStorage"),
    ("ShoppingCenter", "Organization > LocalBusiness > ShoppingCenter"),
    ("SportsActivityLocation", "Organization > LocalBusiness > SportsActivityLocation"),
    ("BowlingAlley", "Organization > LocalBusiness > SportsActivityLocation > BowlingAlley"),
    ("ExerciseGym", "Organization > LocalBusiness > SportsActivityLocation > ExerciseGym"),
    ("GolfCourse", "Organization > LocalBusiness > SportsActivityLocation > GolfCourse"),
    ("HealthClub", "Organization > LocalBusiness > SportsActivityLocation > HealthClub"),
    ("PublicSwimmingPool", "Organization > LocalBusiness > SportsActivityLocation > PublicSwimmingPool"),
    ("SkiResort", "Organization > LocalBusiness > SportsActivityLocation > SkiResort"),
    ("SportsClub", "Organization > LocalBusiness > SportsActivityLocation > SportsClub"),
    ("StadiumOrArena", "Organization > LocalBusiness > SportsActivityLocation > StadiumOrArena"),
    ("TennisComplex", "Organization > LocalBusiness > SportsActivityLocation > TennisComplex"),
    ("Store", "Organization > LocalBusiness > Store"),
    ("AutoPartsStore", "Organization > LocalBusiness > Store > AutoPartsStore"),
    ("BikeStore", "Organization > LocalBusiness > Store > BikeStore"),
    ("BookStore", "Organization > LocalBusiness > Store > BookStore"),
    ("ClothingStore", "Organization > LocalBusiness > Store > ClothingStore"),
    ("ComputerStore", "Organization > LocalBusiness > Store > ComputerStore"),
    ("ConvenienceStore", "Organization > LocalBusiness > Store > ConvenienceStore"),
    ("DepartmentStore", "Organization > LocalBusiness > Store > DepartmentStore"),
    ("ElectronicsStore", "Organization > LocalBusiness > Store > ElectronicsStore"),
    ("Florist", "Organization > LocalBusiness > Store > Florist"),
    ("FurnitureStore", "Organization > LocalBusiness > Store > FurnitureStore"),
    ("GardenStore", "Organization > LocalBusiness > Store > GardenStore"),
    ("GroceryStore", "Organization > LocalBusiness > Store > GroceryStore"),
    ("HardwareStore", "Organization > LocalBusiness > Store > HardwareStore"),
    ("HobbyShop", "Organization > LocalBusiness > Store > HobbyShop"),
    ("HomeGoodsStore", "Organization > LocalBusiness > Store > HomeGoodsStore"),
    ("JewelryStore", "Organization > LocalBusiness > Store > JewelryStore"),
    ("LiquorStore", "Organization > LocalBusiness > Store > LiquorStore"),
    ("MensClothingStore", "Organization > LocalBusiness > Store > MensClothingStore"),
    ("MobilePhoneStore", "Organization > LocalBusiness > Store > MobilePhoneStore"),
    ("MovieRentalStore", "Organization > LocalBusiness > Store > MovieRentalStore"),
    ("MusicStore", "Organization > LocalBusiness > Store > MusicStore"),
    ("OfficeEquipmentStore", "Organization > LocalBusiness > Store > OfficeEquipmentStore"),
    ("OutletStore", "Organization > LocalBusiness > Store > OutletStore"),
    ("PawnShop", "Organization > LocalBusiness > Store > PawnShop"),
    ("PetStore", "Organization > LocalBusiness > Store > PetStore"),
    ("ShoeStore", "Organization > LocalBusiness > Store > ShoeStore"),
    ("SportingGoodsStore", "Organization > LocalBusiness > Store > SportingGoodsStore"),
    ("TireShop", "Organization > LocalBusiness > Store > TireShop"),
    ("ToyStore", "Organization > LocalBusiness > Store > ToyStore"),
    ("WholesaleStore", "Organization > LocalBusiness > Store > WholesaleStore"),
    ("TelevisionStation", "Organization > LocalBusiness > TelevisionStation"),
    ("TouristInformationCenter", "Organization > LocalBusiness > TouristInformationCenter"),
    ("TravelAgency", "Organization > LocalBusiness > TravelAgency"),
    ("MedicalOrganization", "Organization > MedicalOrganization"),
    ("Dentist", "Organization > MedicalOrganization > Dentist"),
    ("Hospital", "Organization > MedicalOrganization > Hospital"),
    ("Pharmacy", "Organization > MedicalOrganization > Pharmacy"),
    ("Physician", "Organization > MedicalOrganization > Physician"),
    ("NGO", "Organization > NGO"),
    ("PerformingGroup", "Organization > PerformingGroup"),
    ("DanceGroup", "Organization > PerformingGroup > DanceGroup"),
    ("MusicGroup", "Organization > PerformingGroup > MusicGroup"),
    ("TheaterGroup", "Organization > PerformingGroup > TheaterGroup"),
    ("SportsOrganization", "Organization > SportsOrganization"),
    ("SportsTeam", "Organization > SportsOrganization > SportsTeam"),
)

SCHEMA_ACTION_CHOICES = (
    ("OrderAction", "OrderAction"),
    ("ReserveAction", "ReserveAction"),
)

SCHEMA_RESULT_CHOICES = (
    ("Reservation", "Reservation"),
    ("BusReservation", "BusReservation"),
    ("EventReservation", "EventReservation"),
    ("FlightReservation", "FlightReservation"),
    ("FoodEstablishmentReservation", "FoodEstablishmentReservation"),
    ("LodgingReservation", "LodgingReservation"),
    ("RentalCarReservation", "RentalCarReservation"),
    ("ReservationPackage", "ReservationPackage"),
    ("TaxiReservation", "TaxiReservation"),
    ("TrainReservation", "TrainReservation"),
)
# fmt: on



# Generated by Django 3.1.1 on 2020-09-04 02:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
    ]

    operations = [
        migrations.CreateModel(
            name='SeoSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('og_meta', models.BooleanField(default=True, help_text='Show an optimized preview when linking to this site on social media. See https://ogp.me/', verbose_name='Use Open Graph Markup')),
                ('twitter_meta', models.BooleanField(default=True, help_text='Shows content as a card when linking to this site on Twitter. See https://developer.twitter.com/en/docs/twitter-for-websites/cards', verbose_name='Use Twitter Markup')),
                ('twitter_site', models.CharField(blank=True, help_text='The @username of the website owner’s Twitter handle.', max_length=16, verbose_name='Twitter Account')),
                ('struct_meta', models.BooleanField(default=True, help_text='Optimizes information about your organization for search engines. See https://schema.org/', verbose_name='Use Structured Data')),
                ('amp_pages', models.BooleanField(default=True, help_text='Generates an alternate AMP version of article pages that are preferred by search engines. See https://amp.dev/', verbose_name='Use AMP Pages')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
            ],
            options={
                'verbose_name': 'SEO',
            },
        ),
    ]



import json
from datetime import datetime
from enum import Enum
from typing import Optional

from django.db import models
from django.utils.translation import gettext_lazy as _
from wagtail.admin.panels import (
    HelpPanel,
    FieldPanel,
    MultiFieldPanel,
)
from wagtail.contrib.settings.models import register_setting
from wagtail.fields import StreamField
from wagtail.models import Page
from wagtail.images import get_image_model_string
from wagtail.images.models import AbstractImage

from wagtailseo import settings, schema, utils
from wagtailseo.blocks import OpenHoursBlock, StructuredDataActionBlock

from wagtail.contrib.settings.models import BaseSiteSetting
from wagtail.admin.widgets.slug import SlugInput


class SeoType(Enum):
    ARTICLE = "article"
    WEBSITE = "website"


class TwitterCard(Enum):
    APP = "app"
    LARGE = "summary_large_image"
    PLAYER = "player"
    SUMMARY = "summary"


class SeoMixin(Page):
    """
    Contains fields for SEO-related attributes on a Page model.
    """

    class Meta:
        abstract = True

    canonical_url = models.URLField(
        blank=True,
        max_length=255,
        verbose_name=_("Canonical URL"),
        help_text=_("Leave blank to use the page's URL."),
    )
    og_image = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Preview image"),
        help_text=_("Shown when linking to this page on social media."),
    )
    struct_org_type = models.CharField(
        default="",
        blank=True,
        max_length=255,
        choices=schema.SCHEMA_ORG_CHOICES,
        verbose_name=_("Organization type"),
        help_text=_("If blank, no structured data will be used on this page."),
    )
    struct_org_name = models.CharField(
        default="",
        blank=True,
        max_length=255,
        verbose_name=_("Organization name"),
        help_text=_("Leave blank to use the site name in Settings > Sites"),
    )
    struct_org_logo = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Organization logo"),
        help_text=_("Leave blank to use the logo in Settings > Layout > Logo"),
    )
    struct_org_image = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Photo of Organization"),
        help_text=_(
            "A photo of the facility. This photo will be cropped to 1:1, 4:3, "
            "and 16:9 aspect ratios automatically."
        ),
    )
    struct_org_phone = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Telephone number"),
        help_text=_(
            "Include country code for best results. For example: +1-216-555-8000"
        ),
    )
    struct_org_address_street = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Street address"),
        help_text=_(
            "House number and street. For example, 55 Public Square Suite 1710"
        ),
    )
    struct_org_address_locality = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("City"),
        help_text=_("City or locality. For example, Cleveland"),
    )
    struct_org_address_region = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("State"),
        help_text=_("State, province, county, or region. For example, OH"),
    )
    struct_org_address_postal = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Postal code"),
        help_text=_("Zip or postal code. For example, 44113"),
    )
    struct_org_address_country = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Country"),
        help_text=_(
            "For example, USA. Two-letter ISO 3166-1 alpha-2 country code is "
            "also acceptable https://en.wikipedia.org/wiki/ISO_3166-1"
        ),
    )
    struct_org_geo_lat = models.DecimalField(
        blank=True,
        null=True,
        max_digits=11,
        decimal_places=8,
        verbose_name=_("Geographic latitude"),
    )
    struct_org_geo_lng = models.DecimalField(
        blank=True,
        null=True,
        max_digits=11,
        decimal_places=8,
        verbose_name=_("Geographic longitude"),
    )
    struct_org_hours = StreamField(
        [("hours", OpenHoursBlock())],
        blank=True,
        verbose_name=_("Hours of operation"),
        use_json_field=True,
    )
    struct_org_actions = StreamField(
        [("actions", StructuredDataActionBlock())],
        blank=True,
        verbose_name=_("Actions"),
        use_json_field=True,
    )
    struct_org_extra_json = models.TextField(
        blank=True,
        verbose_name=_("Additional Organization markup"),
        help_text=_(
            "Additional JSON-LD inserted into the Organization dictionary. "
            "Must be properties of https://schema.org/Organization or the "
            "selected organization type."
        ),
    )

    # The content type of this page, for search engines.
    seo_content_type = SeoType.WEBSITE

    # List of text attribute names on this model, in order of preference,
    # for use as the SEO description.
    seo_description_sources = [
        "search_description",  # Comes from wagtail.Page
    ]

    # List of text attribute names on tthis model, in order of
    # preference, for use as Canonial URL.
    canonical_url_sources = [
        "canonical_url",
    ]

    # List of Image object attribute names on this model, in order of
    # preference, for use as the preferred Open Graph / SEO image.
    seo_image_sources = [
        "og_image",
    ]

    # List of text attribute names on this model, in order of preference,
    # for use as the SEO title.
    seo_pagetitle_sources = [
        "seo_title",  # Comes from wagtail.Page
    ]

    # The style of Twitter card to show.
    seo_twitter_card = TwitterCard.SUMMARY

    @property
    def seo_author(self) -> str:
        """
        Gets the name of the author of this page.
        Override in your Page model as necessary.
        """
        if self.owner:
            return self.owner.get_full_name()
        return ""

    @property
    def seo_canonical_url(self) -> str:
        """
        Gets the full/absolute/canonical URL preferred for meta tags and search engines.
        Override in your Page model as necessary.
        """
        for attr in self.canonical_url_sources:
            if hasattr(self, attr):
                url = getattr(self, attr)
                if url:
                    return url
        return self.get_full_url()

    @property
    def seo_description(self) -> str:
        """
        Gets the correct search engine and Open Graph description of this page.
        Override in your Page model as necessary.
        """
        for attr in self.seo_description_sources:
            if hasattr(self, attr):
                text = getattr(self, attr)
                if text:
                    return text
        return ""

    @property
    def seo_image(self) -> Optional[AbstractImage]:
        """
        Gets the primary Open Graph image of this page.
        """
        for attr in self.seo_image_sources:
            if hasattr(self, attr):
                image = getattr(self, attr)
                if isinstance(image, AbstractImage):
                    return image
        return None

    @property
    def seo_image_url(self) -> str:
        """
        Gets the absolute URL for the primary Open Graph image of this page.
        """
        if self.seo_image:
            url = self.seo_image.get_rendition("original").url
            base_url = utils.get_absolute_media_url(self.get_site())
            return utils.ensure_absolute_url(url, base_url)
        return ""

    @property
    def seo_logo(self) -> Optional[AbstractImage]:
        """
        Gets the primary logo of the organization.
        """
        if self.struct_org_logo:
            return self.struct_org_logo
        return None

    @property
    def seo_logo_url(self) -> str:
        """
        Gets the absolute URL for the organization logo.
        """
        if self.seo_logo:
            url = self.seo_logo.get_rendition("original").url
            base_url = utils.get_absolute_media_url(self.get_site())
            return utils.ensure_absolute_url(url, base_url)
        return ""

    @property
    def seo_og_type(self) -> str:
        """
        Gets the correct Open Graph type for this page.
        Override in your Page model as necessary.
        """
        return self.seo_content_type.value

    @property
    def seo_sitename(self) -> str:
        """
        Gets the site name.
        Override in your Page model as necessary.
        """
        s = self.get_site()
        if s:
            return s.site_name
        return ""

    @property
    def seo_pagetitle(self) -> str:
        """
        Gets the correct search engine and Open Graph title of this page.
        Override in your Page model as necessary.
        """
        for attr in self.seo_pagetitle_sources:
            if hasattr(self, attr):
                text = getattr(self, attr)
                if text:
                    return text

        # Fallback to wagtail.Page.title plus site name.
        return "{0} {1} {2}".format(
            self.title, settings.get("WAGTAILSEO_SEP"), self.seo_sitename
        )

    @property
    def seo_published_at(self) -> datetime:
        """
        Gets the date this page was first published.
        Override in your Page model as necessary.
        """
        return self.first_published_at

    @property
    def seo_twitter_card_content(self) -> str:
        """
        Gets the correct style of twitter card for this page.
        Override in your Page model as necessary.
        """
        return self.seo_twitter_card.value

    @property
    def seo_struct_org_name(self) -> str:
        """
        Gets org name for sturctured data using a fallback.
        """
        if self.struct_org_name:
            return self.struct_org_name
        return self.seo_sitename

    @property
    def seo_struct_org_base_dict(self) -> dict:
        """
        Gets generic "Organization" data for use as a subset of other
        structured data types (for example, as publisher of an Article).

        See: https://developers.google.com/search/docs/data-types/article
        """

        # Base info.
        sd_dict: dict = {
            "@context": "http://schema.org",
            "@type": "Organization",
            "url": self.seo_canonical_url,
            "name": self.seo_struct_org_name,
        }

        # Logo.
        if self.seo_logo:
            sd_dict.update(
                {
                    "logo": {
                        "@type": "ImageObject",
                        "url": self.seo_logo_url,
                    },
                }
            )

        # Image.
        if self.struct_org_image:
            images = utils.get_struct_data_images(
                self.get_site(), self.struct_org_image
            )
            sd_dict.update({"image": images})

        # Telephone.
        if self.struct_org_phone:
            sd_dict.update({"telephone": self.struct_org_phone})

        # Address.
        if self.struct_org_address_street:
            sd_dict.update(
                {
                    "address": {
                        "@type": "PostalAddress",
                        "streetAddress": self.struct_org_address_street,
                        "addressLocality": self.struct_org_address_locality,
                        "addressRegion": self.struct_org_address_region,
                        "postalCode": self.struct_org_address_postal,
                        "addressCountry": self.struct_org_address_country,
                    },
                }
            )

        return sd_dict

    @property
    def seo_struct_org_base_json(self) -> str:
        return json.dumps(self.seo_struct_org_base_dict, cls=utils.StructDataEncoder)

    @property
    def seo_struct_org_dict(self) -> dict:
        """
        Gets full "Organization" structured data on top of base organization data.

        See: https://developers.google.com/search/docs/data-types/local-business
        """

        # Base info.
        sd_dict = self.seo_struct_org_base_dict

        # Override org type to use specific type.
        if self.struct_org_type:
            sd_dict.update({"@type": self.struct_org_type})

        # Geo coordinates.
        if self.struct_org_geo_lat and self.struct_org_geo_lng:
            sd_dict.update(
                {
                    "geo": {
                        "@type": "GeoCoordinates",
                        "latitude": float(self.struct_org_geo_lat),
                        "longitude": float(self.struct_org_geo_lng),
                    },
                }
            )

        # Hours of operation.
        if self.struct_org_hours:
            hours = []
            for spec in self.struct_org_hours:
                hours.append(spec.value.struct_dict)
            sd_dict.update({"openingHoursSpecification": hours})

        # Actions.
        if self.struct_org_actions:
            actions = []
            for action in self.struct_org_actions:
                actions.append(action.value.struct_dict)
            sd_dict.update({"potentialAction": actions})

        # Extra JSON.
        if self.struct_org_extra_json:
            sd_dict.update(json.loads(self.struct_org_extra_json))

        return sd_dict

    @property
    def seo_struct_org_json(self) -> str:
        return json.dumps(self.seo_struct_org_dict, cls=utils.StructDataEncoder)

    @property
    def seo_struct_publisher_dict(self) -> Optional[dict]:
        """
        Gets the base organization info from either this page, or the root page.
        """
        if self.struct_org_type:
            return self.seo_struct_org_base_dict
        else:
            root_page = self.get_site().root_page.specific
            if hasattr(root_page, "seo_struct_org_base_dict"):
                return root_page.seo_struct_org_base_dict
        return None

    @property
    def seo_struct_article_dict(self) -> dict:
        sd_dict = {
            "@context": "http://schema.org",
            "@type": "Article",
            "mainEntityOfPage": {
                "@type": "WebPage",
                "@id": self.seo_canonical_url,
            },
            "headline": self.title,
            "description": self.seo_description,
            "datePublished": self.seo_published_at,
            "dateModified": self.last_published_at,
            "author": {
                "@type": "Person",
                "name": self.seo_author,
            },
        }

        # Image, if available.
        if self.seo_image:
            sd_dict.update(
                {"image": utils.get_struct_data_images(self.get_site(), self.seo_image)}
            )

        # Publisher, if available.
        if self.seo_struct_publisher_dict:
            sd_dict.update({"publisher": self.seo_struct_publisher_dict})

        return sd_dict

    @property
    def seo_struct_article_json(self) -> str:
        return json.dumps(self.seo_struct_article_dict, cls=utils.StructDataEncoder)

    seo_meta_panels = [
        MultiFieldPanel(
            [
                FieldPanel("slug", widget=SlugInput),
                FieldPanel("seo_title"),
                FieldPanel("search_description"),
                FieldPanel("canonical_url"),
                FieldPanel("og_image"),
            ],
            _("Search and Social Previews"),
        ),
    ]

    seo_menu_panels = [
        MultiFieldPanel(
            [
                FieldPanel("show_in_menus"),
            ],
            _("Navigation"),
        ),
    ]

    seo_struct_panels = [
        MultiFieldPanel(
            [
                HelpPanel(
                    heading=_("About Organization Structured Data"),
                    content=_(schema.SCHEMA_HELP),
                ),
                FieldPanel("struct_org_type"),
                FieldPanel("struct_org_name"),
                FieldPanel("struct_org_logo"),
                FieldPanel("struct_org_image"),
                FieldPanel("struct_org_phone"),
                FieldPanel("struct_org_address_street"),
                FieldPanel("struct_org_address_locality"),
                FieldPanel("struct_org_address_region"),
                FieldPanel("struct_org_address_postal"),
                FieldPanel("struct_org_address_country"),
                FieldPanel("struct_org_geo_lat"),
                FieldPanel("struct_org_geo_lng"),
                FieldPanel("struct_org_hours"),
                FieldPanel("struct_org_actions"),
                FieldPanel("struct_org_extra_json"),
            ],
            _("Structured Data - Organization"),
        ),
    ]

    seo_panels = seo_meta_panels + seo_menu_panels + seo_struct_panels


@register_setting(icon="wagtailseo-line-chart")
class SeoSettings(BaseSiteSetting):
    """
    Toggle Search engine optimization features and meta tags.
    """

    class Meta:
        verbose_name = _("SEO")

    og_meta = models.BooleanField(
        default=True,
        verbose_name=_("Use Open Graph Markup"),
        help_text=_(
            "Show an optimized preview when linking to this site on social media. "
            "See https://ogp.me/"
        ),
    )
    twitter_meta = models.BooleanField(
        default=True,
        verbose_name=_("Use Twitter Markup"),
        help_text=_(
            "Shows content as a card when linking to this site on Twitter. "
            "See https://developer.twitter.com/en/docs/twitter-for-websites/cards"
        ),
    )
    twitter_site = models.CharField(
        max_length=16,
        blank=True,
        verbose_name=_("Twitter Account"),
        help_text=_("The @username of the website owner’s Twitter handle."),
    )
    struct_meta = models.BooleanField(
        default=True,
        verbose_name=_("Use Structured Data"),
        help_text=_(
            "Optimizes information about your organization for search engines. "
            "See https://schema.org/"
        ),
    )

    @property
    def at_twitter_site(self):
        """
        The Twitter site handle, prepended with "@".
        """
        handle = self.twitter_site.lstrip("@")
        return "@{0}".format(handle)

    panels = [
        MultiFieldPanel(
            [
                FieldPanel("og_meta"),
                FieldPanel("struct_meta"),
                FieldPanel("twitter_meta"),
                FieldPanel("twitter_site"),
            ],
            heading=_("Search Engine Optimization"),
        )
    ]



import json

from django import forms
from django.utils.translation import gettext_lazy as _
from wagtail import blocks

from wagtailseo import schema


class OpenHoursValue(blocks.StructValue):
    """
    Renders OpenHours in Structured Data format.
    """

    @property
    def struct_dict(self) -> dict:
        """
        Returns a dictionary in structured data format.
        """
        return {
            "@type": "OpeningHoursSpecification",
            "dayOfWeek": self["days"],
            "opens": self["start_time"],  # |date:'H:i'
            "closes": self["end_time"],  # |date:'H:i'
        }


class OpenHoursBlock(blocks.StructBlock):
    """
    Holds day and time combination for business open hours.
    """

    class Meta:
        label = _("Open Hours")
        value_class = OpenHoursValue

    days = blocks.MultipleChoiceBlock(
        required=True,
        verbose_name=_("Days"),
        help_text=_(
            "For late night hours past 23:59, define each day in a separate block."
        ),
        widget=forms.CheckboxSelectMultiple,
        choices=[
            ("Monday", _("Monday")),
            ("Tuesday", _("Tuesday")),
            ("Wednesday", _("Wednesday")),
            ("Thursday", _("Thursday")),
            ("Friday", _("Friday")),
            ("Saturday", _("Saturday")),
            ("Sunday", _("Sunday")),
        ],
    )
    start_time = blocks.TimeBlock(
        verbose_name=_("Opening time"),
    )
    end_time = blocks.TimeBlock(
        verbose_name=_("Closing time"),
    )


class StructuredDataActionValue(blocks.StructValue):
    """
    Renders Action in Structured Data format.
    """

    @property
    def struct_dict(self) -> dict:
        sd_dict = {
            "@type": self["action_type"],
            "target": {
                "@type": "EntryPoint",
                "urlTemplate": self["target"],
                "inLanguage": self["language"],
                "actionPlatform": [
                    "http://schema.org/DesktopWebPlatform",
                    "http://schema.org/IOSPlatform",
                    "http://schema.org/AndroidPlatform",
                ],
            },
        }
        if self["result_type"]:
            sd_dict.update(
                {
                    "result": {
                        "@type": self["result_type"],
                        "name": self["result_name"],
                    }
                }
            )
        if self["extra_json"]:
            sd_dict.update(
                json.loads(self["extra_json"]),
            )
        return sd_dict


class StructuredDataActionBlock(blocks.StructBlock):
    """
    Action object from schema.org
    """

    class Meta:
        label = _("Action")
        value_class = StructuredDataActionValue

    action_type = blocks.ChoiceBlock(
        verbose_name=_("Action Type"),
        required=True,
        choices=schema.SCHEMA_ACTION_CHOICES,
    )
    target = blocks.URLBlock(
        verbose_name=_("Target URL"),
    )
    language = blocks.CharBlock(
        verbose_name=_("Language"),
        help_text=_(
            "If the action is offered in multiple languages, create separate "
            "actions for each language."
        ),
        default="en-US",
    )
    result_type = blocks.ChoiceBlock(
        required=False,
        verbose_name=_("Result Type"),
        help_text=_("Leave blank for OrderAction"),
        choices=schema.SCHEMA_RESULT_CHOICES,
    )
    result_name = blocks.CharBlock(
        required=False,
        verbose_name=_("Result Name"),
        help_text=_('Example: "Reserve a table", "Book an appointment", etc.'),
    )
    extra_json = blocks.RawHTMLBlock(
        required=False,
        verbose_name=_("Additional action markup"),
        classname="monospace",
        help_text=_(
            "Additional JSON-LD inserted into the Action dictionary. "
            "Must be properties of https://schema.org/Action."
        ),
    )



import re
from datetime import date, datetime, time
from json import JSONEncoder
from typing import List, Union

from django.conf import settings
from wagtail.models import Site
from wagtail.images.models import AbstractImage


# Matches a protocol, such as https://
PROTOCOL_RE = re.compile(r"^(\w[\w\.\-\+]*:)*//")
MEDIA_IS_ABSOLUTE = PROTOCOL_RE.match(settings.MEDIA_URL)


def serialize_date(date: Union[date, datetime, time]) -> str:
    """
    Serializes a datetime or time into ISO 8601 format required for Open Graph
    and Structured Data.

    :param Union[date, datetime, time] date: The date object to serialize.
    :rtype: str
    :returns: String-ified date.
    """
    return date.isoformat()


def get_absolute_media_url(site: Site) -> str:
    """
    Returns an absolute base URL for media files.

    This will normally be the site's root URL, except for when MEDIA_URL already
    looks like a full URL (e.g. if media is stored and served from S3, a CDN,
    etc.). The return value can always be safely prefixed to an image URL.

    :param Site site: The site object from which the media is served.
    :rtype: str
    :returns: The absolute base URL for media files of this site.
    """
    if MEDIA_IS_ABSOLUTE:
        return ""

    return site.root_url


def ensure_absolute_url(url: str, base_url: str) -> str:
    if not PROTOCOL_RE.match(url):
        url = base_url + url
    return url


def get_struct_data_images(site: Site, image: AbstractImage) -> List[str]:
    """
    Google requires multiple different aspect ratios for certain structured
    data image fields. This will render the image in 1:1, 4:3, and 16:9 aspect
    ratios with very high resolution and return a list of URLs.

    :param Site site: The Wagtail Site this image belongs to.
    :param Image image: An image descending from Wagtail AbstractImage model.
    :rtype: List[str]
    :return: A list of absolute image URLs.
    """

    base_url = get_absolute_media_url(site)

    # Use huge numbers because Wagtail will not upscale, but will max out at the
    # image's original resolution using the specified aspect ratio.
    # Google wants them high resolution.
    img1x1 = ensure_absolute_url(image.get_rendition("fill-10000x10000").url, base_url)
    img4x3 = ensure_absolute_url(image.get_rendition("fill-40000x30000").url, base_url)
    img16x9 = ensure_absolute_url(image.get_rendition("fill-16000x9000").url, base_url)

    return [img1x1, img4x3, img16x9]


class StructDataEncoder(JSONEncoder):
    """
    Serializes data into LD+JSON format required for Structured Data.
    """

    def default(self, obj):

        # Serialize dates to ISO 8601 format.
        if isinstance(obj, date):
            return serialize_date(obj)

        # Serialize datetimes to ISO 8601 format.
        if isinstance(obj, datetime):
            return serialize_date(obj)

        # Serialize times to ISO 8601 format.
        if isinstance(obj, time):
            return serialize_date(obj)

        # Fallback to default encoding.
        return super().default(obj)



"""
Provide default Django settings.
"""

from django.conf import settings

# Title sitename separator. Default is em-dash.
DEFAULTS = {"WAGTAILSEO_SEP": "—"}


def get(name: str):
    return getattr(settings, name, DEFAULTS[name])



from setuptools import setup
from wagtailseo import __version__

with open("README.md", encoding="utf8") as readme_file:
    readme = readme_file.read()

setup(
    name="cjkcms-seo",
    version=__version__,
    author="Grzegorz Krol",
    author_email="info@cjkcms.com",
    url="https://github.com/cjkpl/cjkcms-seo.git",
    description="Search engine and social media optimization for Wagtail.",
    long_description=readme,
    long_description_content_type="text/markdown",
    license="BSD license",
    include_package_data=True,
    packages=["wagtailseo"],
    python_requires=">=3.8",
    install_requires=["wagtail>=4.0,<=6.0"],
    classifiers=[
        "Framework :: Django",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 4",
        "Framework :: Wagtail :: 5",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
)



"""
Registers wagtail-seo icon in the admin dashboard.
"""
from wagtail import hooks


@hooks.register("register_icons")
def register_icons(icons):
    """
    Add custom SVG icons to the Wagtail admin.
    """
    # These SVG files should be in the django templates folder, and follow exact
    # specifications to work with Wagtail:
    # https://github.com/wagtail/wagtail/pull/6028
    icons.append("wagtailseo/wagtailseo-line-chart.svg")
    return icons



# Generated by Django 3.2.5 on 2021-07-31 17:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailseo', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='seosettings',
            name='amp_pages',
        ),
    ]
